#include <windows.h>
#include <tchar.h>
#include <d3d9.h>
#include <d3dx9.h>

IDirect3D9				*pD3D = 0;
IDirect3DDevice9		*pD3DDevice = 0;
IDirect3DVertexBuffer9	*pVB = 0;
IDirect3DTexture9		*pTexture = 0;

HRESULT hr;

struct Vertex
{
    FLOAT x, y, z;
	FLOAT u, v;
};
#define FVF_VERTEX ( D3DFVF_XYZ | D3DFVF_TEX1 )

int num_v = 3;

LRESULT WINAPI WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_DESTROY:
			PostQuitMessage( 0 );
			return 0;
	}

	return DefWindowProc( hWnd, msg, wParam, lParam );
}


void Render()
{
	//	Czyszczenie ekranu i bufora gbi:
	pD3DDevice->Clear(
		0,
		NULL,
		D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
		D3DCOLOR_XRGB( 50, 40, 30 ),
		1.0f,
		0 );

	hr = pD3DDevice->BeginScene();
	if( FAILED( hr ) )
		return;


	static int rotation;

	//	Macierz wiata:
    D3DXMATRIXA16 matWorld;
    D3DXMatrixIdentity( &matWorld );
    D3DXMatrixRotationY( &matWorld, ++rotation / 200.0f );
    pD3DDevice->SetTransform( D3DTS_WORLD, &matWorld );

	//	Macierz widoku:
	D3DXMATRIXA16 matView;
    D3DXVECTOR3 vEyePt( 0.0f, 1.0f,-5.0f );
    D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );
    pD3DDevice->SetTransform( D3DTS_VIEW, &matView );

	//	Macierz projekcji:
    D3DXMATRIXA16 matProj;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI / 3.0f, 1.0f, 1.0f, 100.0f );
    pD3DDevice->SetTransform( D3DTS_PROJECTION, &matProj );

	//	Wybr tekstury:
	pD3DDevice->SetTexture( 0, pTexture );

	//	Wybr bufora wierzchokw:
	pD3DDevice->SetStreamSource( 0, pVB, 0, sizeof( Vertex ) );

	//	Wybr uywanej struktury wierzchoka:
	pD3DDevice->SetFVF( FVF_VERTEX );

	//	Rysowanie trjktw:
	pD3DDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, num_v/3 );


	pD3DDevice->EndScene();

	pD3DDevice->Present( NULL, NULL, NULL, NULL );
}


int WINAPI _tWinMain( HINSTANCE hInstance, HINSTANCE, LPTSTR, INT )
{
	// Tworzenie okna:
	WNDCLASSEX wc;
	ZeroMemory( &wc, sizeof( WNDCLASSEX ) );
	wc.cbSize = sizeof( WNDCLASSEX );
	wc.style = CS_CLASSDC;
	wc.lpfnWndProc = WndProc;
	wc.hInstance = hInstance;
	wc.lpszClassName = _T( "LabiryntWndClass" );
	RegisterClassEx( &wc );

	HWND hWnd = CreateWindow(
		_T( "LabiryntWndClass" ),
		_T( "Trjwymiarowy labirynt ;)" ),
		WS_SYSMENU | WS_MINIMIZEBOX,
		CW_USEDEFAULT, CW_USEDEFAULT,
		800, 600,
		NULL, NULL,
		hInstance,	
		NULL );



	//	Direct3D:
	pD3D = Direct3DCreate9( D3D_SDK_VERSION );
	if(!pD3D) goto WinMain_end;

    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof( d3dpp ) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

	
    hr = pD3D->CreateDevice(
		D3DADAPTER_DEFAULT,
		D3DDEVTYPE_HAL,
		hWnd,
		D3DCREATE_HARDWARE_VERTEXPROCESSING,
		&d3dpp,
		&pD3DDevice );
	if( FAILED( hr ) ) goto WinMain_end;


	//	Wierzchoki:
	hr = pD3DDevice->CreateVertexBuffer(
		num_v * sizeof( Vertex ),
		0,
		FVF_VERTEX,
		D3DPOOL_DEFAULT,
		&pVB,
		NULL );
	if( FAILED( hr ) ) goto WinMain_end;

	Vertex* pVertices;
    hr = pVB->Lock( 0, 0, ( void** )&pVertices, 0 );
	if( FAILED( hr ) ) goto WinMain_end;

	pVertices[0].x = -1.0f;
	pVertices[0].y = 0.0f;
	pVertices[0].z = 0.0f;
	pVertices[0].u = 0.0f;
	pVertices[0].v = 0.0f;

	pVertices[1].x = 1.0f;
	pVertices[1].y = 0.0f;
	pVertices[1].z = 0.0f;
	pVertices[1].u = 1.0f;
	pVertices[1].v = 0.0f;

	pVertices[2].x = 0.0f;
	pVertices[2].y = 1.73f;
	pVertices[2].z = 0.0f;
	pVertices[2].u = 0.5f;
	pVertices[2].v = 1.0f;

	pVB->Unlock();

    pD3DDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
    pD3DDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
    pD3DDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

	D3DXCreateTextureFromFile( pD3DDevice, _T("tekstura.bmp"), &pTexture );

	
	//	Wywietl okno:
	ShowWindow( hWnd, SW_SHOWDEFAULT );


	//	Ptla komunikatw i renderowania:
	MSG msg;
	for(;;)
	{
		if( !PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
		{
			Render();
			continue;
		}
		
		if(msg.message == WM_QUIT)
			break;

		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}



	//	Do tego bloku mona wej tylko poprzez instrukcj goto:
	if(0)
	{
		WinMain_end:
		msg.wParam = 1;
	}

	//	Zwalniamy interfejsy Direct3D:
	if( pTexture ) pTexture->Release();
	if( pVB ) pVB->Release();
	if( pD3DDevice ) pD3DDevice->Release();
    if( pD3D ) pD3D->Release();

	//	Wyrejestrowanie klasy okna:
	UnregisterClass( _T("LabiryntWndClass"), hInstance );

	//	Zwracamy warto komunikatu WM_QUIT lub 1, jeli wykonane zostao goto:
	return msg.wParam;
}



